use argh::FromArgs;
use bluetooth::BluetoothFeature;
use docker::DockerFeature;
use firewall::FirewallFeature;
use gpu::GPUFeature;
use ollama::OllamaFeature;
use ssh::SSHFeature;
use virt::{VirtFeature, VirtGuestFeature};
use zram::ZramFeature;

pub mod bluetooth;
pub mod docker;
pub mod firewall;
pub mod gpu;
pub mod ollama;
pub mod ssh;
pub mod virt;
pub mod zram;

pub struct OptionSet {
    values: Vec<(String, String)>,
}

impl OptionSet {
    pub fn new(values: Vec<String>) -> Self {
        Self {
            values: values
                .into_iter()
                .map(|x| {
                    let s = x.split_once('=').unwrap();
                    (s.0.to_owned(), s.1.to_owned())
                })
                .collect(),
        }
    }

    pub fn get(&self, key: &str) -> Option<&String> {
        self.values.iter().find(|x| x.0 == key).map(|x| &x.1)
    }

    pub fn get_many(&self, key: &str) -> Vec<String> {
        self.values
            .iter()
            .filter_map(|x| if x.0 == key { Some(x.1.clone()) } else { None })
            .collect()
    }

    pub fn get_flag(&self, key: &str) -> Option<bool> {
        if let Some(val) = self.get(key) {
            if val == "true" || val == "1" {
                return Some(true);
            } else {
                return Some(false);
            }
        }

        None
    }
}

#[derive(FromArgs)]
#[argh(subcommand)]
pub enum Features {
    Bluetooth(BluetoothFeature),
    GPU(GPUFeature),
    Docker(DockerFeature),
    Firewall(FirewallFeature),
    Ollama(OllamaFeature),
    SSH(SSHFeature),
    Virt(VirtFeature),
    VirtGuest(VirtGuestFeature),
    Zram(ZramFeature),
}
